use sea_orm_migration::prelude::*;

#[derive(DeriveMigrationName)]
pub struct Migration;

#[derive(Iden)]
enum MasterPass {
    Table,
    #[iden = "password_hash"]
    PasswordHash,
}

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .alter_table(
                sea_query::Table::alter()
                    .table(MasterPass::Table)
                    .modify_column(
                        ColumnDef::new(MasterPass::PasswordHash)
                            .binary_len(64)
                            .not_null(),
                    )
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .alter_table(
                sea_query::Table::alter()
                    .table(MasterPass::Table)
                    .modify_column(
                        ColumnDef::new(MasterPass::PasswordHash)
                            .binary_len(128)
                            .not_null(),
                    )
                    .to_owned(),
            )
            .await
    }
}
