use crate::prelude::*;
use log::error;
use sea_orm::TransactionTrait;
use tokio::try_join;

/// Gets the master password, deletes the accounts and the master password from DB.
/// Although it doesn't use the master password, we get it to be sure that it's the user who used that command
#[inline]
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    mut ids: MessageIds,
    _: String,
) -> crate::Result<()> {
    dialogue.exit().await?;
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let txn = db.begin().await?;
    let result = try_join!(
        Account::delete_all(user_id, &txn),
        MasterPass::remove(user_id, &txn),
    );
    let text = match result {
        Ok(_) => {
            txn.commit().await?;
            "Everything was deleted"
        }
        Err(err) => {
            error!("{}", crate::Error::from(err));
            txn.rollback().await?;
            "Something went wrong. Try again later"
        }
    };
    ids.alter_message(&bot, text, deletion_markup(), None)
        .await?;
    Ok(())
}

first_handler!(
    delete_all,
    "Send master password to delete EVERYTHING.\nTHIS ACTION IS IRREVERSIBLE",
    State::GetMasterPass,
    get_master_pass
);
