use crate::prelude::*;
use futures::future;
use std::fmt::Write;
use teloxide::types::ParseMode;

/// Handles /get_accounts command by sending the list of copyable account names to the user
#[inline]
pub async fn get_accounts(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
) -> crate::Result<()> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let mut account_names = Account::get_names(user_id, &db).await?;
    let mut text = match account_names.try_next().await? {
        Some(name) => format!("Accounts:\n`{name}`"),
        None => {
            bot.send_message(msg.chat.id, "No accounts found")
                .reply_markup(deletion_markup())
                .await?;
            return Ok(());
        }
    };
    account_names
        .map_err(crate::Error::from)
        .try_for_each(|name| {
            let result = write!(text, "\n`{name}`").map_err(Into::into);
            future::ready(result)
        })
        .await?;
    bot.send_message(msg.chat.id, text)
        .parse_mode(ParseMode::MarkdownV2)
        .reply_markup(deletion_markup())
        .await?;
    Ok(())
}
