use crate::prelude::*;
use cryptography::passwords::{check_master_pass, PasswordValidity};

#[inline]
fn process_validity(validity: PasswordValidity) -> Result<(), String> {
    if validity.is_empty() {
        return Ok(());
    }

    let mut error_text = "Your master password is invalid:\n".to_owned();

    if validity.contains(PasswordValidity::NO_LOWERCASE) {
        error_text.push_str("\n* It doesn't have any lowercase characters")
    }
    if validity.contains(PasswordValidity::NO_UPPERCASE) {
        error_text.push_str("\n* It doesn't have any uppercase characters")
    }
    if validity.contains(PasswordValidity::NO_NUMBER) {
        error_text.push_str("\n* It doesn't have any numbers")
    }
    if validity.contains(PasswordValidity::NO_SPECIAL_CHARACTER) {
        error_text.push_str("\n* It doesn't have any special characters")
    }
    if validity.contains(PasswordValidity::TOO_SHORT) {
        error_text.push_str("\n* It is shorter than 8 characters")
    }

    error_text.push_str("\n\nModify your password and send it again");

    Err(error_text)
}

/// Checks that the account with that name exists
#[inline]
async fn check_new_master_pass(
    _: &Message,
    _: &DatabaseConnection,
    password: &str,
) -> crate::Result<Option<String>> {
    let validity = check_master_pass(password);

    Ok(process_validity(validity).err())
}

crate::simple_state_handler!(
    get_new_master_pass,
    check_new_master_pass,
    "Couldn't get the text of the message. Send the master password again"
);
