use crate::prelude::*;

/// Validates a new account
#[inline]
async fn check_new_account_name(
    msg: &Message,
    db: &DatabaseConnection,
    name: &str,
) -> crate::Result<Option<String>> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;

    if Account::exists(user_id, name, db).await? {
        Ok(Some("Account already exists".to_owned()))
    } else if !validate_field(name) {
        Ok(Some("Invalid account name. Try again".to_owned()))
    } else {
        Ok(None)
    }
}

crate::simple_state_handler!(
    get_new_name,
    check_new_account_name,
    "Couldn't get the text of the message. Send the name of the new account again"
);
